from .HTTPClient import HTTPClient
from ..utils import is_base64, is_url


class JinaChatClient(HTTPClient):
    def __init__(self, headers=None, options=None):
        baseUrl = 'https://api.chat.jina.ai/v1/chat'
        defaultHeaders = { 
            'Content-Type': 'application/json',
        }
        mergedHeaders = defaultHeaders.update(headers)
        super().__init__(baseUrl=baseUrl, headers=defaultHeaders, options=options)

    def from_array(self, input, options=None):
        return {
            'messages': [
                {
                    'content': i,
                    **(((is_url(i) or is_base64(i)) and { 'image': i }) or {}),
                    'role': 'user',
                    **(options or {})
                }
                for i in input
            ],
            **(options or {})
        }

    def from_string(self, input, options=None):
        return {
            'messages': [
                {
                    'content': input,
                    **(((is_url(input) or is_base64(input)) and { 'image': input }) or {}),
                    'role': 'user',
                    **(options or {})
                }
            ],
            **(options or {})
        }

    def to_simplified_output(self, output):
        if 'choices' not in output or len(output['choices']) < 1 or output['choices'][0]['message']['content'] == '':
            raise Exception('Remote API Error, bad output: ' + str(output))
        return {
            'output': output['choices'][0]['message']['content'],
            'chatId': output['chatId']
        }

    def generate(self, data, options = None):
        raw_output = self.post('/completions', data)
        simplified_output = self.to_simplified_output(raw_output)
        if options and 'raw' in options:
            simplified_output['raw'] = raw_output
        return simplified_output

    def stream(self, data, options = None):
        return self.post('/completions', data, False)



from .clients.SceneXClient import SceneXClient
from .clients.PromptPerfectClient import PromptPerfectClient
from .clients.RationaleClient import RationaleClient
from .clients.JinaChatClient import JinaChatClient
from .clients.BestBannerClient import BestBannerClient
from .utils import is_url, is_base64, image_to_base64

class JinaAI:
    def __init__(self, secrets={}):
        PPSecret = f"token {secrets['promptperfect-secret']}" if secrets and 'promptperfect-secret' in secrets else ''
        SXSecret = f"token {secrets['scenex-secret']}" if secrets and 'scenex-secret' in secrets else ''
        RASecret = f"token {secrets['rationale-secret']}" if secrets and 'rationale-secret' in secrets else ''
        CCSecret = f"Bearer {secrets['jinachat-secret']}" if secrets and 'jinachat-secret' in secrets else ''
        BBSecret = f"token {secrets['bestbanner-secret']}" if secrets and 'bestbanner-secret' in secrets else ''
        self.PPClient = PromptPerfectClient(headers = { "x-api-key": PPSecret })
        self.SXClient = SceneXClient(headers = { "x-api-key": SXSecret })
        self.RAClient = RationaleClient(headers = { "x-api-key": RASecret })
        self.CCClient = JinaChatClient(headers = { "authorization": CCSecret })
        self.BBClient = BestBannerClient(headers = { "x-api-key": BBSecret })

    def decide(self, input, options=None):
        if isinstance(input, list):
            data = self.RAClient.from_array(input, options)
        elif isinstance(input, str):
            data = self.RAClient.from_string(input, options)
        else:
            data = input
        return self.RAClient.decide(data, options)

    def optimize(self, input, options=None):
        if isinstance(input, list):
            data = self.PPClient.from_array(input, options)
        elif isinstance(input, str):
            data = self.PPClient.from_string(input, options)
        else:
            data = input
        return self.PPClient.optimize(data, options)

    def describe(self, input, options=None):
        if isinstance(input, list):
            data = self.SXClient.from_array(input, options)
        elif isinstance(input, str):
            data = self.SXClient.from_string(input, options)
        else:
            data = input
        return self.SXClient.describe(data, options)

    def generate(self, input, options=None):
        if isinstance(input, list):
            data = self.CCClient.from_array(input, options)
        elif isinstance(input, str):
            data = self.CCClient.from_string(input, options)
        else:
            data = input
        if options is not None and options.get('stream', False):
            return self.CCClient.stream(data, options)
        else:
            return self.CCClient.generate(data, options)

    def imagine(self, input, options=None):
        if isinstance(input, list):
            data = self.BBClient.from_array(input, options)
        elif isinstance(input, str):
            data = self.BBClient.from_string(input, options)
        else:
            data = input
        return self.BBClient.imagine(data, options)

    class utils:
        @staticmethod
        def is_url(string):
            return is_url(string)
        @staticmethod
        def is_base64(string):
            return is_base64(string)
        @staticmethod
        def image_to_base64(file_path):
            return image_to_base64(file_path)


from .HTTPClient import HTTPClient

def autoFillFeatures(options=None):
    features = options.get('features', []) if options else []
    if options and 'question' in options and 'question_answer' not in features:
        features.append('question_answer')
    return features

class SceneXClient(HTTPClient):
    def __init__(self, headers=None, options=None):
        baseUrl = 'https://api.scenex.jina.ai/v1'
        defaultHeaders = { 
            'Content-Type': 'application/json',
        }
        mergedHeaders = defaultHeaders.update(headers)
        super().__init__(baseUrl=baseUrl, headers=defaultHeaders, options=options)

    def from_array(self, input, options=None):
        return {
            'data': [
                {
                    'image': i,
                    'features': autoFillFeatures(options),
                    **(options or {})
                }
                for i in input
            ]
        }

    def from_string(self, input, options=None):
        return {
            'data': [
                {
                    'image': input,
                    'features': autoFillFeatures(options),
                    **(options or {})
                }
            ]
        }

    def to_simplified_output(self, output):
        if not output.get('result') or any(x.get('text') and x.get('text') != '' for x in output['result']) is False:
            raise Exception('Remote API Error, bad output: {}'.format(json.dumps(output)))
        return {
            'results': [
                {
                    'output': r['answer'] if 'answer' in r and r['answer'] is not None else r['text'],
                    'i18n': r['i18n']
                }
                for r in output['result']
            ]
        }

    def describe(self, data, options = None):
        raw_output = self.post('/describe', data)
        simplified_output = self.to_simplified_output(raw_output)
        if options and 'raw' in options:
            simplified_output['raw'] = raw_output
        return simplified_output




import requests

class HTTPClient:
    def __init__(self, baseUrl, headers=None, options=None):
        self.baseUrl = baseUrl
        self.headers = headers if headers else {}
        self.options = options if options else {}

    def setHeaders(self, headers):
        self.headers = headers

    def addHeader(self, header):
        self.headers.update(header)

    def get(self, url):
        response = requests.get(self.baseUrl + url, headers=self.headers)
        responseData = response.json()
        return responseData

    def post(self, url, data, toJson=True):
        response = requests.post(self.baseUrl + url, json=data, headers=self.headers, **self.options)
        if toJson == False:
            return response
        else:
            responseData = response.json()
            if "error" in responseData:
                raise Exception(responseData["error"])
            return responseData

    def put(self, url, data):
        response = requests.put(self.baseUrl + url, headers=self.headers)
        responseData = response.json()
        return responseData

    def delete(self, url):
        response = requests.delete(self.baseUrl + url, headers=self.headers)
        responseData = response.json()
        return responseData



from .HTTPClient import HTTPClient

class BestBannerClient(HTTPClient):
    def __init__(self, headers=None, options=None):
        baseUrl = 'https://api.bestbanner.jina.ai/v1'
        defaultHeaders = { 
            'Content-Type': 'application/json',
        }
        mergedHeaders = defaultHeaders.update(headers)
        super().__init__(baseUrl=baseUrl, headers=defaultHeaders, options=options)

    def from_array(self, input, options=None):
        return {
            'data': [
                {
                    'text': i,
                    **(options or {})
                }
                for i in input
            ]
        }

    def from_string(self, input, options=None):
        return {
            'data': [
                {
                    'text': input,
                    **(options or {})
                }
            ]
        }

    def to_simplified_output(self, output):
        if not output.get('result') or any(x.get('banners') and len(x['banners']) != 0 for x in output['result']) is False:
            raise Exception('Remote API Error, bad output: {}'.format(json.dumps(output)))
        return {
            'results': [
                {
                    'output': [
                        b['url'] for b in r['banners']
                    ]
                }
                for r in output['result']
            ]
        }

    def imagine(self, data, options = None):
        raw_output = self.post('/generate', data)
        simplified_output = self.to_simplified_output(raw_output)
        if options and 'raw' in options:
            simplified_output['raw'] = raw_output
        return simplified_output




from .HTTPClient import HTTPClient
from ..utils import is_base64, is_url

MAXLEN = 300

class RationaleClient(HTTPClient):
    def __init__(self, headers=None, options=None):
        baseUrl = 'https://us-central1-rationale-ai.cloudfunctions.net'
        defaultHeaders = { 
            'Content-Type': 'application/json',
        }
        mergedHeaders = defaultHeaders.update(headers)
        super().__init__(baseUrl=baseUrl, headers=defaultHeaders, options=options)

    def from_array(self, input, options=None):
        return {
            'data': [
                {
                    'decision': i[:MAXLEN],
                    **(options or {})
                }
                for i in input
            ]
        }

    def from_string(self, input, options=None):
        return {
            'data': [
                {
                    'decision': input[:MAXLEN],
                    **(options or {})
                }
            ]
        }

    def to_simplified_output(self, output):
        if 'result' not in output or 'result' not in output['result']:
            raise Exception('Remote API Error, bad output: ' + json.dumps(output))
        return {
            'results': [
                {
                    'proscons': r['keyResults'] if r['analysis'] == 'proscons' else None,
                    'swot': r['keyResults'] if r['analysis'] == 'swot' else None,
                    'multichoice': r['keyResults'] if r['analysis'] == 'multichoice' else None,
                    'outcomes': r['keyResults'] if r['analysis'] == 'outcomes' else None,
                }
                for r in output['result']['result']
            ]
        }

    def decide(self, data, options = None):
        raw_output = self.post('/analysisApi', data)
        simplified_output = self.to_simplified_output(raw_output)
        if options and 'raw' in options:
            simplified_output['raw'] = raw_output
        return simplified_output




import base64
import mimetypes
import os
import re

def is_url(string):
    url_pattern = r'^(?:\w+:)?\/\/([^\s.]+\.\S{2}|localhost[:?\d]*)\S*$'
    return bool(re.match(url_pattern, string))

def is_base64(string):
    base64_pattern = r'^data:[A-Za-z0-9+/]+;base64,'
    return bool(re.match(base64_pattern, string))

def image_to_base64(file_path):
    try:
        with open(file_path, 'rb') as file:
            file_data = file.read()
            base64_data = base64.b64encode(file_data).decode('utf-8')
            mime_type = get_mime_type(file_path)
            base64_string = f"data:{mime_type};base64,{base64_data}"
            return base64_string
    except Exception as error:
        raise Exception(f"Image to base64 error: {str(error)}")

def get_mime_type(file_path):
    mime_type, _ = mimetypes.guess_type(file_path)
    return mime_type or 'application/octet-stream'




from setuptools import setup, find_packages
import pathlib

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

setup(
    name='jinaai',
    version='0.2.4',
    author='Jina AI',
    author_email='guillaume.roncari@jina.ai',
    description='Jina AI Python SDK',
    url='https://github.com/jina-ai/jinaai-py.git',
    packages=find_packages(),
    install_requires=[
        'requests',
    ],
    long_description=README,
    long_description_content_type='text/markdown', 
)
